clear
clc
close all

% generate the input signal
x(1:50) = 200; x(51:100)= 0;
x = [x x x x x x x x x x 200*ones(1,24)];

% calculate Matlab's fft
N = length(x);
Y = fft(x);

% now, let's write the FIR
Wn = 0.02; % let's see if we can just get the lowest frequency sin
           % wave
ord = 47; % ord+1 must be a multiple of 4
FIR = fir1(ord,Wn);
% now filter noisy signal:
xfil = filter(FIR,1,x);
Yfil = fft(xfil);

% generate data for FFT plots for the original and filtered signal
mag = 2*abs(Y(1:N/2+1))/N; 
mag(1) = mag(1)/2; 
mag(N/2+1) = mag(N/2+1)/2;

magfil = 2*abs(Yfil(1:N/2+1))/N; 
magfil(1) = magfil(1)/2; 
magfil(N/2+1) = magfil(N/2+1)/2;

freqs = linspace(0,1,N/2+1); 


% now we can read in the values sent from the PIC
DATA = load('temp.txt');
N = length(DATA);
xpic = DATA(:,1);
xfirpic = DATA(:,2);
Xfftpic = DATA(1:N/2+1,3);
Xfftfir = DATA(1:N/2+1,4);

Xfftpic = 2*abs(Xfftpic);
Xfftpic(1) = Xfftpic(1)/2;
Xfftpic(N/2+1) = Xfftpic(N/2+1)/2;

Xfftfir = 2*abs(Xfftfir);
Xfftfir(1) = Xfftfir(1)/2;
Xfftfir(N/2+1) = Xfftfir(N/2+1)/2;

% new we are ready to plot
subplot(3,1,1);
hold on;
title('Plot of the original signal, and the FIR filtered signal')
xlabel('Sample number')
ylabel('Amplitude')
plot(x,'Marker','o');
plot(xfil,'Color','red','LineWidth',2);
plot(xfirpic,'*','Color','black');
hold off;
legend('Original Signal','MATLAB FIR', 'PIC FIR')
axis([-10,1050,-10,210])

subplot(3,1,2);
hold on;
title('FFTs of the original signal')
ylabel('Magnitude')
xlabel('Normalized frequency (fraction of Nyquist Frequency')
stem(freqs,mag)
stem(freqs,Xfftpic,'Color','black')
legend('MATLAB FFT', 'PIC FFT')
hold off;

subplot(3,1,3);
hold on;
title('FFTs of the filtered signal')
ylabel('Magnitude')
xlabel('Normalized frequency (fraction of Nyquist Frequency')
stem(freqs,magfil)
stem(freqs,Xfftfir,'Color','black')
legend('MATLAB FFT', 'PIC FFT')
hold off;
